<?php

/**
 * Tests to make sure an object's related objects and relations to other
 * objects are deleted with the object itself.
 * This is relevant to test, as errors here have been known to make nacoma
 * completely unable to export it's settings.
 */
class DeleteTest extends PHPUnit_Framework_TestCase
{
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		require_once('classes/object_class.php');
		require_once('include/import.inc.php');
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		$this->assertTrue(import_config('phptap', $cfgs), "import_config('phptap', '$cfgs')");
		$IMPORTING = false;

		$this->host = new host_class();
		$this->host->get_default_object();
		$this->host->obj['host_name'] = 'dead object';
		$this->host->obj['alias'] = 'dead object';
		$this->host->obj['address'] = 'address';
		$this->service = new service_class();
		$this->service->get_default_object();
		$this->service->obj['service_description'] = 'dead';
		$this->service->obj['check_command'] = 1;
	}

	function tearDown()
	{
		pdo_exec_query('DELETE FROM management_pack WHERE management_pack_name = "an mgmt pack"');
		evil_global_teardown();
	}

	function relationTest($obj, $rels) {
		$obj_id = $obj->id;
		foreach ($rels as $rel => $cnt) {
			$res = pdo_exec_query('SELECT * FROM '.get_junction_table_name($obj->type, $rel)." WHERE $obj->type=$obj_id");
			$ids = $res->fetchAll(PDO::FETCH_ASSOC);
			$this->assertCount($cnt, $ids, "Unexpected number of $rel relations for {$obj->type} \"{$obj->get_object_name()}\": got ".var_export($ids, true));
		}
	}

	function saveAndTest($obj, $rels=array()) {
		$this->assertTrue($obj->save_object(), 'Failed to save object: '.var_export($obj->validation_error_list, true));
		$obj_id = $obj->id;
		$res = pdo_exec_query("SELECT 1 FROM $obj->db_table WHERE id=$obj_id");
		$this->assertCount(1, $res->fetchAll(PDO::FETCH_NUM), "Expected to find the $obj->type object");
		$this->relationTest($obj, $rels);
	}

	function deleteAndTest($obj, $rels=array()) {
		$obj_id = $obj->id;
		$this->assertTrue($obj->banish_object(), "Couldn't delete object: ".$obj->get_deletion_message());
		$res = pdo_exec_query("SELECT 1 FROM $obj->db_table WHERE id=$obj_id");
		$this->assertEquals(count($res->fetchAll(PDO::FETCH_NUM)), 0, "$obj->type object was deleted");
		foreach ($rels as $rel => $cnt)
			$rels[$rel] = 0;
		$this->relationTest($obj, $rels);
	}

	function testDeleteHostWithRelations()
	{
		$this->host->obj['parents'] = array(36);
		$this->host->obj['hostgroups'] = array(1);
		$this->host->obj['contacts'] = array(1, 2);
		$this->host->obj['contact_groups'] = array(1);
		$this->saveAndTest($this->host, array('contacts' => 2, 'contact_groups' => 1, 'parents' => 1, 'hostgroup'=> 1));
		$this->deleteAndTest($this->host, array('contacts' => 2, 'contact_groups' => 1, 'parents' => 1, 'hostgroup'=> 1));
	}

	function testDeleteServiceWithRelations()
	{
		$member = new host_class('host_9');
		$this->assertTrue(is_numeric($member->id), 'Member host exists');
		$this->service->obj['contacts'] = array(1,2);
		$this->service->obj['contact_groups'] = array(1);
		$this->service->obj['servicegroups'] = array(1);
		$this->service->obj['host_name'] = $member->id;
		$this->saveAndTest($this->service, array('contacts' => 2, 'contact_groups' => 1, 'servicegroup' => 1));
		$this->deleteAndTest($this->service, array('contacts' => 2, 'contact_groups' => 1, 'servicegroup' => 1));
	}

	function testDeleteHostGroupWithRelations()
	{
		$member = new host_class('host_9');
		$this->assertTrue(is_numeric($member->id), 'Member host exists');
		$hg = new hostgroup_class();
		$hg->obj['hostgroup_name'] = 'name';
		$hg->obj['alias'] = 'name';
		$hg->obj['members'] = array($member->id);
		$this->assertTrue($hg->save_object(), 'Hostgroup should save, got '.var_export($hg->validation_error_list, true));
		$this->relationTest($hg, array('members' => 1), 'Only one member should be created for new hostgroup');
		$this->host->obj['hostgroups'] = array($hg->id);
		$this->assertTrue($this->host->save_object(), 'Host should be saved, got '.var_export($this->host->validation_error_list, true));
		$hg = new hostgroup_class($hg->id);
		$this->saveAndTest($hg, array('members' => 2));
		$this->deleteAndTest($hg, array('members' => 2));

		$this->host->banish_object();
	}

	function testDeleteHostWithService()
	{
		$this->saveAndTest($this->host);
		$this->service->obj['host_name'] = $this->host->id;
		$this->service->obj['contacts'] = array(1, 2);
		$this->service->obj['contact_groups'] = array(1);
		$this->saveAndTest($this->service, array('contacts'=>2, 'contact_groups'=>1));
		$this->host->banish_object();
		$res = pdo_exec_query("SELECT 1 FROM service WHERE service_description = 'dead'");
		$this->assertEmpty($res->fetchAll(PDO::FETCH_NUM), 'Service is gone');
		$this->relationTest($this->service, array('contact'=>0,'contactgroup'=>0));
	}

	function testDeleteHostWithServiceEscalation()
	{
		$this->saveAndTest($this->host);
		$this->service->obj['host_name'] = $this->host->id;
		$this->saveAndTest($this->service);
		$sesc = new serviceescalation_class();
		$sesc->get_default_object();
		$sesc->obj['service'] = $this->service->id;
		$sesc->obj['contacts'] = array(1,2);
		$this->saveAndTest($sesc, array('contacts'=>2));
		$this->host->banish_object();
		$res = pdo_exec_query("SELECT 1 FROM service WHERE id=".$this->service->id);
		$this->assertEmpty($res->fetchAll(PDO::FETCH_NUM), 'Service is gone');
		unset($res);
		$res = pdo_exec_query("SELECT 1 FROM serviceescalation WHERE id=".$sesc->id);
		$this->assertEmpty($res->fetchAll(PDO::FETCH_NUM), 'Serviceescalation is gone');
		$this->relationTest($sesc, array('contacts'=>0));
	}

	function testDeleteWithReqAltDeps()
	{
		$hg = new hostgroup_class();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->obj['members'] = array(43);
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");
		$this->service->set_master_object_type('hostgroup');
		$this->service->obj['hostgroup_name'] = $hg->id;
		$this->service->save_object();
		$this->assertEmpty($this->service->validation_error_list, 'New service saved ok');
		$this->assertTrue($hg->banish_object(), "Deleting hostgroup should be OK, got".$hg->delete_message);
		$res = pdo_exec_query('SELECT COUNT(1) FROM service WHERE id = '.$this->service->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->assertEquals(0, $row[0], "Service was deleted when it's parent object was");
	}

	function testDeleteHostDeps()
	{
		$hg = new hostgroup_class();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->obj['members'] = array(43);
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");

		$host = new host_class();
		$host->get_default_object();
		$host->obj['host_name'] = 'test object';
		$host->obj['alias'] = 'test object';
		$host->obj['address'] = 'address';
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'New host saved ok');

		$this->host->save_object();
		$this->assertEmpty($this->host->validation_error_list, 'New host saved ok');

		$hd = new hostdependency_class();
		$hd->get_default_object();
		$hd->obj['host_name'] = $host->id;
		$hd->obj['hostgroup_name'] = $hg->id;
		$hd->obj['dependent_host_name'] = $this->host->id;
		$hd->save_object();
		$this->assertEmpty($hd->validation_error_list, 'New hostdependency saved ok');

		$hgobj = $hg->obj;
		$hg->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->assertTrue(!empty($nhd->obj), 'Hostdependency still exists');
		$res = pdo_exec_query('SELECT host_name, hostgroup_name FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_ASSOC);
		$this->assertEmpty($row['hostgroup_name'], 'Hostgroup cleared in dependency when deleted');
		$this->assertEquals($row['host_name'], $host->id, 'Host not cleared in dependency');
		unset($res);

		$hostobj = $host->obj;
		$host->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->assertEmpty($nhd->obj, "Hostdependency doesn't still exist");
		$res = pdo_exec_query('SELECT COUNT(1) FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->assertEquals($row[0], 0, "Hostgroup deleted when both it's req_alts are");
		unset($res);

		$hg->set_object_id('new');
		$hg->obj = $hgobj;
		$hg->save_object();
		$hd->obj['hostgroup_name'] = $hg->id;
		$host->set_object_id('new');
		$host->obj = $hostobj;
		$host->save_object();
		$hd->obj['host_name'] = $host->id;
		$hd->save_object();
		$this->assertEmpty($hd->validation_error_list, 'Hostdependency updated ok');

		$hostobj = $host->obj;
		$host->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->assertTrue(!empty($nhd->obj), 'Hostdependency still exists');
		$res = pdo_exec_query('SELECT host_name, hostgroup_name FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_ASSOC);
		$this->assertEmpty($row['host_name'], 'Host cleared in dependency when deleted');
		$this->assertEquals($row['hostgroup_name'], $hg->id, 'Hostgroup not cleared in dependency');
		unset($res);

		$hg->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->assertEmpty($nhd->obj, "Hostdependency doesn't still exist");
		$res = pdo_exec_query('SELECT COUNT(1) FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->assertEquals($row[0], 0, "Hostdependency deleted when both it's req_alts are");
		unset($res);
		$this->host->banish_object();
	}

	function testDeleteReasonText()
	{
		$hg = new hostgroup_class();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");

		$host = new host_class();
		$host->obj = $this->host->obj;
		$host->obj['hostgroups'] = array($hg->id);
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'New host saved ok');

		$this->assertSame($host->delete_object(), false, "Deleting host with relation shouldn't work without warnings");
		$this->assertSame(preg_match('/(affect the following objects).+(hostgroup test hostgroup)/', $host->delete_message), 1, "Expected 'test hostgroup' to be mentioned as a related object, got: $host->delete_message");
		$this->assertSame(preg_match('/(completely remove the following objects)/', $host->delete_message), 0, "Expected no slave objects, got: $host->delete_message");

		$service = new service_class();
		$service->obj = $this->service->obj;
		$service->obj['host_name'] = $host->id;
		$service->save_object();
		$this->assertEmpty($service->validation_error_list, 'New host service ok');

		$this->assertSame($host->delete_object(), false, "Deleting host with slave shouldn't work without warnings");
		$this->assertSame(preg_match('/(completely remove the following objects).+(service dead object;dead).+(affect the following objects).+(hostgroup test hostgroup)/', $host->delete_message), 1, "Expected 'test hostgroup' to be mentioned as a related object and 'dead' slave to be mentioned as a slave, got: $host->delete_message");

		$hg->banish_object();
		$host->banish_object();
	}

	function testDeleteMgmtPack()
	{
		$hg = new hostgroup_class();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->save_object();
		$this->assertEmpty($hg->validation_error_list, "New hostgroup saved ok");

		$mgmt = new management_pack_class();
		$mgmt->get_default_object();
		$mgmt->obj['management_pack_name'] = 'an mgmt pack';
		$mgmt->obj['hostgroup'] = $hg->obj['hostgroup_name'];
		$p = $mgmt->get_properties();
		$mgmt->obj['icon'] = current($p['special']['icon']);
		$mgmt->obj['description'] = 'the pack has a purpose';
		$this->assertTrue($mgmt->save_object(), 'Saving new management pack should be ok, got '.var_export($mgmt->validation_error_list, true));
		$this->assertEmpty($mgmt->validation_error_list, "There should not be errors saving new management pack");

		$this->assertSame($hg->delete_object(), false, "Deleting a the hostgroup with a mgmt pack should fail");
		$this->assertSame(preg_match('/(You may not delete this hostgroup).+(an mgmt pack)/', $hg->delete_message), 1, "When deleting hostgroup with mgmtpack, the mgmtpack should block deletion: ".$hg->delete_message);

		$this->assertSame($hg->banish_object(), false, "Banishing a hostgroup with a mgmt pack should fail");
		$this->assertSame($mgmt->banish_object(), true, "Banishing a mgmt pack should work");
		$this->assertSame($hg->banish_object(), true, "Banishing a hostgroup without a mgmt pack should work");
	}
}

?>
